# Generated by Django 3.2 on 2021-07-09 18:10

import django.core.validators
from django.db import migrations
import jutil.modelfields
import re


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0024_account_notes"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="number",
            field=jutil.modelfields.SafeCharField(
                blank=True,
                db_index=True,
                default="",
                error_messages={"invalid": "Only accept letters, numbers, underscores or hyphens."},
                max_length=32,
                validators=[
                    django.core.validators.RegexValidator(
                        re.compile("^[-a-zA-Z0-9_]+\\Z"),
                        "Enter a valid “slug” consisting of letters, numbers, underscores or hyphens.",
                        "invalid",
                    )
                ],
                verbose_name="invoice number",
            ),
        ),
    ]
