# Generated by Django 3.2.6 on 2022-02-20 02:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0026_alter_invoice_number"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="paid_amount",
            field=models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name="paid amount"),
        ),
    ]
