{% extends "admin/change_form.html" %}

{% load i18n admin_urls static admin_modify %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% translate 'Home' %}</a>
    &rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
    {% if original %}
        &rsaquo; <a href="{% url 'admin:jacc_account_change' original.account.id %}">{{ original.account }}</a>
        &rsaquo; <a href="{% url 'admin:jacc_accountentry_account_changelist' original.account.id %}">{% trans "account entries"|capfirst %}</a>
    {% endif %}
    {% if has_view_permission %}
        {% if not original and not service_agreement %}
            &rsaquo; <a href="{% url opts|admin_urlname:'changelist' %}">{{ opts.verbose_name_plural|capfirst }}</a>
        {% endif %}
    {% else %}
        &rsaquo; {{ opts.verbose_name_plural|capfirst }}
    {% endif %}
&rsaquo; {% if add %}{% blocktranslate with name=opts.verbose_name %}Add {{ name }}{% endblocktranslate %}{% else %} {{ original|truncatewords:"18" }}{% endif %}
</div>
{% endblock %}
