# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-10-30 19:35
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0003_auto_20171030_1933"),
    ]

    operations = [
        migrations.RenameModel(old_name="AccountEntrySource", new_name="AccountEntrySourceFile"),
    ]
