# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-02 03:17
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0005_auto_20171030_1958"),
    ]

    operations = [
        migrations.AddField(
            model_name="account",
            name="name",
            field=models.CharField(blank=True, db_index=True, default="", max_length=32, verbose_name="name"),
        ),
    ]
