# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-10-20 13:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="accountentry",
            name="amount",
            field=models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name="amount"),
        ),
    ]
