# Generated by Django 3.2 on 2021-07-12 15:02

from django.db import migrations
import jutil.modelfields


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0025_alter_invoice_number"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="number",
            field=jutil.modelfields.SafeCharField(blank=True, db_index=True, default="", max_length=32, verbose_name="invoice number"),
        ),
    ]
