# Generated by Django 3.1 on 2021-03-11 20:54

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("jacc", "0022_auto_20200629_0346"),
    ]

    operations = [
        migrations.CreateModel(
            name="AccountEntryNote",
            fields=[
                ("id", models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        db_index=True,
                        default=django.utils.timezone.now,
                        editable=False,
                        verbose_name="created",
                    ),
                ),
                ("last_modified", models.DateTimeField(auto_now=True, verbose_name="last modified")),
                ("note", models.TextField(verbose_name="note")),
                (
                    "account_entry",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="note_set",
                        to="jacc.accountentry",
                        verbose_name="account entry",
                    ),
                ),
                (
                    "created_by",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="accountentrynote_set",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="created by",
                    ),
                ),
            ],
            options={
                "verbose_name": "account entry note",
                "verbose_name_plural": "account entry notes",
            },
        ),
    ]
