# Generated by Django 2.0.2 on 2018-02-18 06:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("jacc", "0011_auto_20180208_1146"),
    ]

    operations = [
        migrations.AlterField(
            model_name="accountentry",
            name="amount",
            field=models.DecimalField(
                blank=True,
                db_index=True,
                decimal_places=2,
                default=None,
                max_digits=10,
                null=True,
                verbose_name="amount",
            ),
        ),
        migrations.AlterField(
            model_name="accountentry",
            name="last_modified",
            field=models.DateTimeField(auto_now=True, verbose_name="last modified"),
        ),
    ]
